﻿Module Module1

    Sub Main()
        DoDelegateSamples()
        'DoTrickList()
        Console.ReadLine()
    End Sub

    Private Function GetAnimals() As List(Of CircusAnimal)
        Dim animals As New List(Of CircusAnimal) From {New Lion, New Tiger, New Bear}
        Return animals
    End Function

    Private Sub DoDelegateSamples()
        Dim animals As List(Of CircusAnimal) = GetAnimals()
        animals.AsParallel.ForAll(
            Sub(animal)
                Console.WriteLine(animal.GetType.Name)
                AddHandler animal.RequestIncentive, AddressOf GiveIncentive

                'Instance delegate
                animal.DoTrick(AddressOf animal.Stand)

                'Shared delegate
                animal.DoTrick(AddressOf CircusAnimal.WatchOthers)

                'Overloaded method
                animal.DoTrick(AddressOf animal.Speak, 2)

                'Named delegate
                animal.DoTrick(AddressOf DelegateAction)

                'Anonymous delegate
                animal.DoTrick(Sub() Console.WriteLine("Anonymous Method"))

                ' Lambda expression as a variable
                Dim LiftOneLeg As TrickAction = Sub()
                                                       animal.LegsOnGround -= 1
                                                   End Sub
                animal.DoTrick(LiftOneLeg)

                'Lambda expression inline
                animal.DoTrick(Sub()
                                      animal.LegsOnGround += 3
                                  End Sub)

                Console.WriteLine("Legs now on ground: {0}", animal.LegsOnGround)

                'Multi-line lambda
                animal.DoTrick(Sub()
                                      animal.LegsOnGround = 0
                                      Console.WriteLine("I'm Jumping now!")
                                      animal.LegsOnGround = 4
                                  End Sub)

                'Asynch
                animal.BeginTrick(Sub() Console.WriteLine("Trick Beginning.... " & animal.ToString))

                Console.WriteLine("")

            End Sub)

        Console.WriteLine()
    End Sub

    Private Sub DoTrickList()
        Dim animals As List(Of CircusAnimal) = GetAnimalsWithTrickList()
        animals.ForEach(
            Sub(animal)
                Console.WriteLine(animal.GetType.Name)
                animal.DoTricks()
                Console.WriteLine("Legs now on ground: {0}", animal.LegsOnGround)
                Console.WriteLine("Animal valid: {0}", animal.IsValid)
                Console.WriteLine()
            End Sub)

        'Find invalid anumals
        Console.WriteLine("Invalid: ")
        'For Each animal In animals.Where(Function(item) Not item.IsValid)
        '    Console.WriteLine(animal.ToString)
        'Next

        animals _
            .Where(Function(item) Not item.IsValid).ToList _
            .ForEach(Sub(item) Console.WriteLine(item.ToString))

    End Sub

    Private Sub DelegateAction()
        Console.WriteLine("Performing an action as a named delegate")
    End Sub

    Private Sub GiveIncentive(ByVal sender As Object, ByVal e As EventArgs)
        DirectCast(sender, CircusAnimal).IncentivesGiven += 1
    End Sub

    Private Function GetAnimalsWithTrickList() As List(Of CircusAnimal)
        Dim animals = GetAnimals()
        animals(0).TrickList.Add(AddressOf animals(0).Stand)
        animals(1).TrickList.Add(AddressOf CircusAnimal.WatchOthers)
        animals(2).TrickList.Add(AddressOf animals(2).Speak)
        animals(0).TrickList.Add(AddressOf DelegateAction)
        animals(1).TrickList.Add(Sub() Console.WriteLine("Anonymous Method"))
        animals(2).TrickList.Add(Sub()
                                     animals(2).LegsOnGround += 3
                                 End Sub)
        Return animals
    End Function


End Module
